@@ -23,7 +23,7 @@ which is in turn shorthand for
   test html "my test" (X,Y)
 -}
 
-infix 5 =:
+infix 4 =:
 (=:) :: (ToString a, ToPandoc a)
      => String -> (a, String) -> Test
 (=:) = test html
@@ -40,7 +40,7 @@ tests = [ testGroup "inline code"
           ]
         , testGroup "images"
           [ "alt with formatting" =:
-            image "/url" "title" ("my " +++ emph "image")
+            image "/url" "title" ("my " <> emph "image")
             =?> "<img src=\"/url\" title=\"title\" alt=\"my image\" />"
           ]
         ]
