@@ -215,6 +215,7 @@ pSimpleTable = try $ do
   TagOpen _ _ <- pSatisfy (~== TagOpen "table" [])
   skipMany pBlank
   head' <- option [] $ pOptInTag "thead" $ pInTags "tr" (pCell "th")
+  skipMany pBlank
   rows <- pOptInTag "tbody"
           $ many1 $ try $ skipMany pBlank >> pInTags "tr" (pCell "td")
   skipMany pBlank
@@ -420,8 +421,12 @@ pTagContents =
   pStr <|> pSpace <|> smartPunctuation pTagContents <|> pSymbol <|> pBad
 
 pStr :: GenParser Char ParserState Inline
-pStr = liftM Str $ many1 $ satisfy $ \c ->
-           not (isSpace c) && not (isSpecial c) && not (isBad c)
+pStr = do
+  result <- many1 $ satisfy $ \c ->
+                     not (isSpace c) && not (isSpecial c) && not (isBad c)
+  pos <- getPosition
+  updateState $ \s -> s{ stateLastStrPos = Just pos }
+  return $ Str result
 
 isSpecial :: Char -> Bool
 isSpecial '"' = True
@@ -502,16 +507,35 @@ blockHtmlTags = ["address", "blockquote", "body", "center", "dir", "div",
                  "dt", "frameset", "li", "tbody", "td", "tfoot",
                  "th", "thead", "tr", "script", "style"]
 
+-- We want to allow raw docbook in markdown documents, so we
+-- include docbook block tags here too.
+blockDocBookTags :: [String]
+blockDocBookTags = ["calloutlist", "bibliolist", "glosslist", "itemizedlist",
+                    "orderedlist", "segmentedlist", "simplelist",
+                    "variablelist", "caution", "important", "note", "tip",
+                    "warning", "address", "literallayout", "programlisting",
+                    "programlistingco", "screen", "screenco", "screenshot",
+                    "synopsis", "example", "informalexample", "figure",
+                    "informalfigure", "table", "informaltable", "para",
+                    "simpara", "formalpara", "equation", "informalequation",
+                    "figure", "screenshot", "mediaobject", "qandaset",
+                    "procedure", "task", "cmdsynopsis", "funcsynopsis",
+                    "classsynopsis", "blockquote", "epigraph", "msgset",
+                    "sidebar", "title"]
+
+blockTags :: [String]
+blockTags = blockHtmlTags ++ blockDocBookTags
+
 isInlineTag :: Tag String -> Bool
-isInlineTag t = tagOpen (`notElem` blockHtmlTags) (const True) t ||
-                tagClose (`notElem` blockHtmlTags) t ||
+isInlineTag t = tagOpen (`notElem` blockTags) (const True) t ||
+                tagClose (`notElem` blockTags) t ||
                 tagComment (const True) t
 
 isBlockTag :: Tag String -> Bool
 isBlockTag t = tagOpen (`elem` blocktags) (const True) t ||
                tagClose (`elem` blocktags) t ||
                tagComment (const True) t
-                 where blocktags = blockHtmlTags ++ eitherBlockOrInline
+                 where blocktags = blockTags ++ eitherBlockOrInline
 
 isTextTag :: Tag String -> Bool
 isTextTag = tagText (const True)
@@ -546,8 +570,8 @@ t `closes` t2 |
    t `elem` ["h1","h2","h3","h4","h5","h6","dl","ol","ul","table","div","p"] &&
    t2 `elem` ["h1","h2","h3","h4","h5","h6","p" ] = True -- not "div"
 t1 `closes` t2 |
-   t1 `elem` blockHtmlTags &&
-   t2 `notElem` (blockHtmlTags ++ eitherBlockOrInline) = True
+   t1 `elem` blockTags &&
+   t2 `notElem` (blockTags ++ eitherBlockOrInline) = True
 _ `closes` _ = False
 
 --- parsers for use in markdown, textile readers
