@@ -233,17 +233,19 @@ blockToMarkdown _ HorizontalRule =
 blockToMarkdown opts (Header level inlines) = do
   contents <- inlineListToMarkdown opts inlines
   st <- get
-  -- use setext style headers if in literate haskell mode.
-  -- ghc interprets '#' characters in column 1 as line number specifiers.
-  if writerLiterateHaskell opts || stPlain st
-     then let len = offset contents
-          in  return $ contents <> cr <>
-                       (case level of
-                             1  -> text $ replicate len '='
-                             2  -> text $ replicate len '-'
-                             _  -> empty) <> blankline
-     else return $
-       text ((replicate level '#') ++ " ") <> contents <> blankline
+  let setext = writerSetextHeaders opts
+  return $ nowrap
+         $ case level of
+            1 | setext ->
+                  contents <> cr <> text (replicate (offset contents) '=') <>
+                  blankline
+            2 | setext ->
+                  contents <> cr <> text (replicate (offset contents) '-') <>
+                  blankline
+            -- ghc interprets '#' characters in column 1 as linenum specifiers.
+            _ | stPlain st || writerLiterateHaskell opts ->
+                contents <> blankline
+            _ -> text (replicate level '#') <> space <> contents <> blankline
 blockToMarkdown opts (CodeBlock (_,classes,_) str)
   | "haskell" `elem` classes && "literate" `elem` classes &&
     writerLiterateHaskell opts =
@@ -434,10 +436,6 @@ inlineToMarkdown opts (Quoted SingleQuote lst) = do
 inlineToMarkdown opts (Quoted DoubleQuote lst) = do
   contents <- inlineListToMarkdown opts lst
   return $ "“" <> contents <> "”"
-inlineToMarkdown _ EmDash = return "\8212"
-inlineToMarkdown _ EnDash = return "\8211"
-inlineToMarkdown _ Apostrophe = return "\8217"
-inlineToMarkdown _ Ellipses = return "\8230"
 inlineToMarkdown opts (Code attr str) =
   let tickGroups = filter (\s -> '`' `elem` s) $ group str 
       longest    = if null tickGroups
@@ -495,17 +493,16 @@ inlineToMarkdown opts (Cite (c:cs) lst)
         modekey SuppressAuthor = "-"
         modekey _              = ""
 inlineToMarkdown _ (Cite _ _) = return $ text ""
-inlineToMarkdown opts (Link txt (src', tit)) = do
+inlineToMarkdown opts (Link txt (src, tit)) = do
   linktext <- inlineListToMarkdown opts txt
   let linktitle = if null tit
                      then empty
                      else text $ " \"" ++ tit ++ "\""
-  let src = unescapeURI src'
   let srcSuffix = if isPrefixOf "mailto:" src then drop 7 src else src
-  let useRefLinks = writerReferenceLinks opts
   let useAuto = case (tit,txt) of
                       ("", [Code _ s]) | s == srcSuffix -> True
                       _                                 -> False
+  let useRefLinks = writerReferenceLinks opts && not useAuto
   ref <- if useRefLinks then getReference txt (src, tit) else return []
   reftext <- inlineListToMarkdown opts ref
   return $ if useAuto
