@@ -11,6 +11,7 @@ import qualified Tests.Readers.RST
 import qualified Tests.Writers.ConTeXt
 import qualified Tests.Writers.HTML
 import qualified Tests.Writers.Native
+import qualified Tests.Writers.Markdown
 import qualified Tests.Shared
 
 tests :: [Test]
@@ -20,6 +21,7 @@ tests = [ testGroup "Old" Tests.Old.tests
           [ testGroup "Native" Tests.Writers.Native.tests
           , testGroup "ConTeXt" Tests.Writers.ConTeXt.tests
           , testGroup "HTML" Tests.Writers.HTML.tests
+          , testGroup "Markdown" Tests.Writers.Markdown.tests
           ]
         , testGroup "Readers"
           [ testGroup "LaTeX" Tests.Readers.LaTeX.tests
